@@ -22,27 +22,27 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 '''
 # suppress all warnings.
+import textwrap
+import rich
+import shlex
+from .task import task_backend, task_git, task_git_commit, task_replay, task_fortune
+from . import defaults
+from . import debian
+from . import frontend
+import sys
+import os
+import re
+import argparse
+from prompt_toolkit.completion import Completer, Completion
+from prompt_toolkit import PromptSession
+from rich.panel import Panel
+from rich.markup import escape
+from prompt_toolkit.styles import Style
+from typing import List, Optional
 import warnings
 warnings.filterwarnings("ignore")
 
-from typing import List, Optional
-from prompt_toolkit.styles import Style
-from rich.markup import escape
-from rich.panel import Panel
-from prompt_toolkit import PromptSession
-from prompt_toolkit.completion import Completer, Completion
-import argparse
-import re
-import os
-import sys
-from . import frontend
-from . import debian
-from . import defaults
-from .task import task_backend, task_git, task_git_commit, task_replay, task_fortune
-import shlex
-import rich
 console = rich.get_console()
-import textwrap
 
 
 def version() -> None:
@@ -59,12 +59,81 @@ def generate_config_file(ag) -> None:
     exit(0)
 
 
+def fresh_install_guide(ag) -> None:
+    '''
+    special task: print fresh install guide, then quit
+    '''
+    __doc__ = '''\
+DebGPT Fresh Install Guide
+==========================
+
+To use DebGPT, you need to configure a frontend to connect to the LLM backend.
+There are several frontends available, including:
+
+   (default frontend, commercial, proprietary, need-api-key, OpenAI-API)
+ * `openai`: a frontend that connects to the OpenAI API. You need to obtain an
+   API key from https://platform.openai.com/api-keys and set it in the
+   `--openai_api_key` argument or the `OPENAI_API_KEY` environment variable.
+   Note, this is a commercial proprietary service that needs to be paied for.
+
+   (self-hosted, open-source, OpenAI-API)
+ * `llamafile`: a frontend that connects to the llamafile JSON API service.
+   Please read the instructions at https://github.com/Mozilla-Ocho/llamafile
+   to setup the self-hosted llamafile service.
+
+   (self-hosted, open-source, OpenAI-API)
+ * `ollama`: a frontend that connects to the Ollama JSON API service.
+   Please read the instructions at https://ollama.com/ to setup the
+   self-hosted Ollama service.
+
+   (self-hosted, open-source, need-api-key, OpenAI-API)
+ * `vllm`: a frontend that connects to a vLLM service instance.
+   See https://docs.vllm.ai/en/latest/ for more information.
+   This is a OpenAI-API compatible self-hosted service. You need an API key
+   that matches to the one used in the vLLM service.
+
+   (self-hosted, open-source, transformers, debugging)
+ * `zmq`: a frontend that connects to a self-hosted LLM inference server.
+   This pairs with the self-contained primitive backend implementation
+   of debgpt, which is directly based on `transformers`.
+
+   (debugging, testing)
+ * `dryrun`: a fake frontend that will do nothing other than printing the
+   generated prompt. This is only useful for debugging.
+
+The frontend is specified by the `-F|--frontend` argument. For convenience,
+you can also specify the frontend in the `~/.debgpt/config.toml` file.
+
+You may use the following command to generate a template `config.toml` file:
+
+ $ debgpt genconfig > ~/.debgpt/config.toml
+
+The important flags for each of the frontends are provided in the template.
+
+Enjoy DebGPT!'''
+    console.print(__doc__)
+    exit(0)
+
+
 def parse_args(argv):
     '''
     argparse with subparsers. Generate a config.toml template as byproduct.
     '''
+    # helper functions
+    def __add_arg_to_config(template, parser, argname, formatter: callable = repr):
+        '''
+        We will create a template for the config.toml file, based on the
+        help messages of the argument parser. In that sense I do not have
+        to write everything twice, and this avoids many human errors.
+        '''
+        template += '\n'.join('# ' + x for x in textwrap.wrap(
+            parser._option_string_actions['--'+argname].help))
+        template += f'''\n{argname} = {formatter(getattr(conf, argname))}\n'''
+        return template
+
     # if ~/.debgpt/config.toml exists, parse it to override the built-in defaults.
-    conf = defaults.Config()
+    _verbose = any(x in argv for x in ('-v', '--verbose'))
+    conf = defaults.Config(verbose=_verbose)
     # override the loaded configurations again with command line arguments
     ag = argparse.ArgumentParser()
 
@@ -91,14 +160,19 @@ use Meta+Enter to accept the input instead.')
     ag.add_argument('--debgpt_home', type=str, default=conf['debgpt_home'],
                     help='directory to store cache and sessions.')
     ag.add_argument('--frontend', '-F', type=str, default=conf['frontend'],
-                    choices=('dryrun', 'zmq', 'openai'),
+                    choices=('dryrun', 'zmq', 'openai',
+                             'llamafile', 'ollama', 'vllm'),
                     help=f"default frontend is {conf['frontend']}. Available \
-choices are: (zmq, openai, dryrun). The 'dryrun' is a fake frontend that will \
+choices are: (dryrun, zmq, openai, zmq, llamafile, ollama, vllm).\
+The 'dryrun' is a fake frontend that will \
 do nothing other than printing the generated prompt. So that you can copy \
 it to web-based LLMs in that case.")
-    config_template += '\n'.join('# ' + x for x in textwrap.wrap(
-        ag._option_string_actions['--frontend'].help))
-    config_template += f'''\nfrontend = {repr(conf.frontend)}\n'''
+    config_template = __add_arg_to_config(config_template, ag, 'frontend')
+
+    ag.add_argument('--monochrome', type=bool, default=conf['monochrome'],
+                    help='disable colorized output for prompt_toolkit.')
+    config_template = __add_arg_to_config(config_template, ag, 'monochrome',
+                                          formatter=lambda x: str(x).lower())
 
     # LLM Inference Arguments
     config_template += '''\n
@@ -110,48 +184,85 @@ it to web-based LLMs in that case.")
                     help='''Sampling temperature. Typically ranges within [0,1]. \
 Low values like 0.2 gives more focused (coherent) answer. \
 High values like 0.8 gives a more random (creative) answer. \
-Not suggested to combine this with with --top_p. \
-See https://platform.openai.com/docs/api-reference/chat/create \
+Not suggested to combine this with with --top_p. See \
+https://platform.openai.com/docs/api-reference/ \
     ''')
-    config_template += '\n'.join('# ' + x for x in textwrap.wrap(
-        ag._option_string_actions['--temperature'].help))
-    config_template += f'''\ntemperature = {repr(conf.temperature)}\n'''
+    config_template = __add_arg_to_config(config_template, ag, 'temperature')
 
     ag.add_argument('--top_p', '-P', type=float, default=conf['top_p'])
     # TODO: add this in config template
 
     # Specific to OpenAI Frontend
     config_template += '''\n
-##############################
+#############################
 # Specific to OpenAI Frontend
-##############################
+#############################
 \n'''
     ag.add_argument('--openai_base_url', type=str,
                     default=conf['openai_base_url'],
                     help='OpenAI API is a widely adopted standard. You can \
 switch to other compatible service providers, or a self-hosted compatible \
 server.')
-    config_template += '\n'.join('# ' + x for x in textwrap.wrap(
-        ag._option_string_actions['--openai_base_url'].help))
-    config_template += f'''\nopenai_base_url = {repr(conf.openai_base_url)}\n'''
-    config_template += '\n'
+    config_template = __add_arg_to_config(
+        config_template, ag, 'openai_base_url')
 
     ag.add_argument('--openai_api_key', type=str,
                     default=conf['openai_api_key'],
                     help='API key is necessary to access services including \
 OpenAI API server. https://platform.openai.com/api-keys')
-    config_template += '\n'.join('# ' + x for x in textwrap.wrap(
-        ag._option_string_actions['--openai_api_key'].help))
-    config_template += f'''\nopenai_api_key = {repr(conf.openai_api_key)}\n'''
-    config_template += '\n'
+    config_template = __add_arg_to_config(
+        config_template, ag, 'openai_api_key')
 
     ag.add_argument('--openai_model', type=str, default=conf['openai_model'],
                     help='For instance, gpt-3.5-turbo (4k context), \
 gpt-3.5-turbo-16k (16k context), gpt-4, gpt-4-32k (32k context). \
 Their prices vary. See https://platform.openai.com/docs/models .')
-    config_template += '\n'.join('# ' + x for x in textwrap.wrap(
-        ag._option_string_actions['--openai_model'].help))
-    config_template += f'''\nopenai_model = {repr(conf.openai_model)}\n'''
+    config_template = __add_arg_to_config(config_template, ag, 'openai_model')
+
+    # Specific to Llamafile Frontend
+    config_template += '''\n
+################################
+# Specific to Llamafile Frontend
+################################
+\n'''
+    ag.add_argument('--llamafile_base_url', type=str, default=conf['llamafile_base_url'],
+                    help='the URL to the llamafile JSON API service.')
+    config_template = __add_arg_to_config(
+        config_template, ag, 'llamafile_base_url')
+
+    # Specific to Ollama Frontend
+    config_template += '''\n
+#########################################################
+# Specific to Ollama Frontend (OpenAI compatibility mode)
+#########################################################
+\n'''
+    ag.add_argument('--ollama_base_url', type=str, default=conf['ollama_base_url'],
+                    help='the URL to the Ollama JSON API service.')
+    config_template = __add_arg_to_config(
+        config_template, ag, 'ollama_base_url')
+
+    ag.add_argument('--ollama_model', type=str, default=conf['ollama_model'],
+                    help='the model to use in Ollama. For instance, llama3.2')
+    config_template = __add_arg_to_config(config_template, ag, 'ollama_model')
+
+    # Specific to vLLM Frontend
+    config_template += '''\n
+###########################
+# Specific to vLLM Frontend
+###########################
+\n'''
+    ag.add_argument('--vllm_base_url', type=str, default=conf['vllm_base_url'],
+                    help='the URL to the vllm JSON API service.')
+    config_template = __add_arg_to_config(config_template, ag, 'vllm_base_url')
+
+    ag.add_argument('--vllm_api_key', type=str,
+                    default=conf['vllm_api_key'],
+                    help='vLLM API key is necessary to access services')
+    config_template = __add_arg_to_config(config_template, ag, 'vllm_api_key')
+
+    ag.add_argument('--vllm_model', type=str, default=conf['vllm_model'],
+                    help='the model to use in vllm. For instance, llama3.2')
+    config_template = __add_arg_to_config(config_template, ag, 'vllm_model')
 
     # Specific to ZMQ Frontend
     config_template += '''\n
@@ -161,9 +272,7 @@ Their prices vary. See https://platform.openai.com/docs/models .')
 \n'''
     ag.add_argument('--zmq_backend', type=str, default=conf['zmq_backend'],
                     help='the ZMQ backend URL that the frontend will connect to')
-    config_template += '\n'.join('# ' + x for x in textwrap.wrap(
-        ag._option_string_actions['--zmq_backend'].help))
-    config_template += f'''\nzmq_backend = {repr(conf.zmq_backend)}\n'''
+    config_template = __add_arg_to_config(config_template, ag, 'zmq_backend')
 
     # Prompt Loaders (numbered list). You can specify them multiple times.
     # for instance, `debgpt -H -f foo.py -f bar.py`.
@@ -209,7 +318,8 @@ Their prices vary. See https://platform.openai.com/docs/models .')
                     + "Or a customized string not starting with the colon.")
 
     # Task Specific Subparsers
-    subps = ag.add_subparsers(help='specific task handling')
+    subps = ag.add_subparsers(dest='subparser_name',
+                              help='specific task handling')
     ag.set_defaults(func=lambda ag: None)  # if no subparser is specified
 
     # Specific to ZMQ Backend (self-hosted LLM Inference)
@@ -277,12 +387,14 @@ def parse_args_order(argv) -> List[str]:
     For example, we need to match
     -f, --file, -Hf (-[^-]*f), into --file
     '''
-    order : List[str] = []
+    order: List[str] = []
+
     def _match_ls(probe: str, long: str, short: str, dest: List[str]):
         if any(probe == x for x in (long, short)) \
                 or any(probe.startswith(x+'=') for x in (long, short)) \
                 or re.match(r'-[^-]*'+short[-1], probe):
             dest.append(long.lstrip('--'))
+
     def _match_l(probe: str, long: str, dest: List[str]):
         if probe == long or probe.startswith(long+'='):
             dest.append(long.lstrip('--'))
@@ -358,8 +470,11 @@ def gather_information_ordered(msg: Optional[str], ag, ag_order) -> Optional[str
 
 def interactive_mode(f: frontend.AbstractFrontend, ag):
     # create prompt_toolkit style
-    prompt_style = Style([('prompt', 'bold fg:ansibrightcyan'),
-                          ('', 'bold ansiwhite')])
+    if ag.monochrome:
+        prompt_style = Style([])
+    else:
+        prompt_style = Style([('prompt', 'bold fg:ansibrightcyan'),
+                              ('', 'bold ansiwhite')])
 
     # Completer with several keywords keywords to be completed
     class CustomCompleter(Completer):
@@ -419,6 +534,11 @@ def main(argv=sys.argv[1:]):
         exit(0)
     if ag.verbose:
         console.log(ag)
+    # detect first-time launch (fresh install)
+    if ag.frontend == 'openai' and ag.openai_api_key == 'your-openai-api-key' \
+        and ag.openai_base_url == 'https://api.openai.com/v1':
+        fresh_install_guide(ag)
+        exit(0)
 
     # parse argument order
     ag_order = parse_args_order(argv)
