@@ -27,6 +27,7 @@ try:
 except:
     import pip._vendor.tomli as tomllib  # for python < 3.10
 import rich
+console = rich.get_console()
 
 
 ########################
@@ -38,31 +39,48 @@ CONFIG = os.path.join(HOME, 'config.toml')
 
 
 class Config(object):
-    def __init__(self, home: str = HOME, config: str = CONFIG):
+    def __init__(self, home: str = HOME, config: str = CONFIG, verbose: bool = False):
         # The built-in defaults will be overridden by config file
         self.toml = {
             # CLI/Frontend Bebavior
             'frontend': 'openai',
             'debgpt_home': HOME,
+            'monochrome': False,
             # LLM Inference Parameters
             'temperature': 0.5,
             'top_p': 1.0,
             # OpenAI Frontend Specific
             'openai_base_url': 'https://api.openai.com/v1',
-            'openai_model': 'gpt-3.5-turbo',
-            'openai_api_key': 'empty',
+            'openai_model': 'gpt-4o',
+            'openai_api_key': 'your-openai-api-key',
+            # Llamafile Frontend Specific
+            'llamafile_base_url': 'http://localhost:8080/v1',
+            # Ollama Frontend Specific
+            'ollama_base_url': 'http://localhost:11434/v1',
+            'ollama_model': 'llama3.2',
+            # vLLM Frontend Specific
+            'vllm_base_url': 'http://localhost:8000/v1',
+            'vllm_api_key': 'your-vllm-api-key',
+            'vllm_model': 'NousResearch/Meta-Llama-3-8B-Instruct',
             # ZMQ Frontend Specific
             'zmq_backend': 'tcp://localhost:11177',
         }
         # the built-in defaults will be overridden by config file
         if not os.path.exists(home):
+            if verbose:
+                rich.print(f'Creating directory {home}')
             os.mkdir(home)
         if os.path.exists(config):
+            if verbose:
+                rich.print(f'Loading configuration from {config}')
             with open(config, 'rb') as f:
                 content = tomllib.load(f)
                 self.toml.update(content)
         # some arguments will be overrden by environment variables
         if (openai_api_key := os.getenv('OPENAI_API_KEY', None)) is not None:
+            if verbose:
+                rich.print(
+                    f'Found environment variable OPENAI_API_KEY. Overriding openai_api_key')
             self.toml['openai_api_key'] = openai_api_key
         # all the above will be overridden by command line arguments
         pass
